#!/usr/bin/perl -w
use pbbfGlobal;
use POSIX;
use strict;

my $doEps_ = 1;

my $testPre_ = "PbbfComm";

my $numTests_ = 3;

my ($pKey_, $qKey_, $rKey_);
$pKey_ = "p";
$qKey_ = "q";
$rKey_ = "r";

my (@xKey_, @param1Key_, @param2Key_);
$xKey_[0] = $pKey_;
$param1Key_[0] = $qKey_;
$param2Key_[0] = $rKey_;

$xKey_[1] = $qKey_;
$param1Key_[1] = $pKey_;
$param2Key_[1] = $rKey_;

$xKey_[2] = $rKey_;
$param1Key_[2] = $pKey_;
$param2Key_[2] = $qKey_;

# x parameters for tests
# Have to use qw for decimals.  Otherwise, 1.0 will look like
# 1 for hash key lookups and will fail
my %protVals_;
$protVals_{$pKey_}[0] = [qw(0 3 7 10)];
$protVals_{$qKey_}[0] = [qw(0 10)];
$protVals_{$rKey_}[0] = [qw(0 10)];

$protVals_{$pKey_}[1] = [qw(0 10)];
$protVals_{$qKey_}[1] = [qw(0 3 7 10)];
$protVals_{$rKey_}[1] = [qw(0 10)];

$protVals_{$pKey_}[2] = [qw(0 10)];
$protVals_{$qKey_}[2] = [qw(0 10)];
$protVals_{$rKey_}[2] = [qw(0 3 7 10)];

my @xLabels_;
$xLabels_[0] = "p";
$xLabels_[1] = "q";
$xLabels_[2] = "r";

my @plotTypes_ = qw(energy latency pct_recv 
	total_recv total_sent);

my %yLabels_;
$yLabels_{"energy"} = 
	"Fraction of Time Spent Not Sleeping"; 
$yLabels_{"latency"} = 
	"Average Broadcast Latency (ms)"; 
$yLabels_{"pct_recv"} = 
	"Unique Data Packets Received/\\nTotal ".
	"Unique Data Packets Sent at Source";
$yLabels_{"total_recv"} = 
	"Total Data Packets Received/\\nTotal ".
	"Unique Data Packets Sent at Source"; 
$yLabels_{"total_sent"} = 
	"Total Data Packets Sent/\\nTotal ".
	"Unique Data Packets Sent at Source"; 

my $TOTAL_BCASTS = 11;
# TinyOS time measurements have a frequency
# of 921.6 MHz, so they're not quite microseconds
my $LAT_CONV = 921.6;
my $ON_TIME_BASE = 255;
my $NUM_PROB_LEVELS = 10;

sub main;
sub plotTests;
sub promptUser;

sub avgHash;

main();

sub main {

	print "\n";

	my $promptVal = &promptUser("Are you sure that you want ".
			"to plot tests (y/n)? ", "n");

	if($promptVal eq "y" || $promptVal eq "Y") {
		plotTests();
	}

	print "\nFinished!!!\n\n";

}

sub plotTests {

	my $gnuplotDir = $pbbfGlobal::GNUPLOT_DIR;
	my $statsDir = $pbbfGlobal::STATS_DIR;
	my (@statsFiles, @gpFiles);
	local (*STATS_OUT);
	my ($f, $tmp, $tmp2, $tmp3);
	my ($i,$j);
	my ($curP, $curQ, $curR, $curNode);
	my (%fracOnTimeVals, %avgLatVals, %totalAppRecvVals,
		%totalDataRecvVals, %totalDataSentVals, 
		%totalDataResentVals);

	@gpFiles = glob($gnuplotDir.$testPre_."*");
	foreach $f (@gpFiles) {
		unlink $f;
	}

	@statsFiles = glob($statsDir.$testPre_."*");
	foreach $f (@statsFiles) {

		$curP = $curQ = $curR = $curNode = undef;

		open(STATS_OUT, $f) || die "Cannot open $f\n";
		while(<STATS_OUT>) {

			if(($tmp,$tmp2,$tmp3) = 
					/.*P=(\d+), Q=(\d+), R=(\d+).*/) {
				$curP = $tmp;	
				$curQ = $tmp2;	
				$curR = $tmp3;	
				#print "p: $curP, q: $curQ, r: $curR\n";
			} elsif(($tmp) = /^Node (\d+)/) {
				$curNode = $tmp;
			} elsif(($tmp) = /TotalDataSent (\d+)/) {
				$tmp2 = $totalDataResentVals{$curP}{$curQ}
					{$curR}{$curNode};
				$tmp2 = (defined $tmp2) ? $tmp2 : 0;

				$totalDataSentVals{$curP}{$curQ}
					{$curR}{$curNode} = (($tmp  / $TOTAL_BCASTS) + $tmp2);
			} elsif(($tmp) = /TotalDataResent (\d+)/) {
				$totalDataResentVals{$curP}{$curQ}
					{$curR}{$curNode} = ($tmp / $TOTAL_BCASTS);

				$tmp2 = $totalDataSentVals{$curP}{$curQ}
					{$curR}{$curNode};
				if(defined $tmp2) {
					$totalDataSentVals{$curP}{$curQ}
						{$curR}{$curNode} = (($tmp  / $TOTAL_BCASTS) + $tmp2);
				}
			} elsif(($tmp) = /TotalDataRecv (\d+)/) {
				$totalDataRecvVals{$curP}{$curQ}
					{$curR}{$curNode} = ($tmp / $TOTAL_BCASTS);
			} elsif(($tmp) = /TotalAppRecv (\d+)/) {
				$totalAppRecvVals{$curP}{$curQ}
					{$curR}{$curNode} = ($tmp / $TOTAL_BCASTS);
			} elsif(($tmp) = /AvgLat (\d+)/) {
				$avgLatVals{$curP}{$curQ}
					{$curR}{$curNode} = ($tmp / $LAT_CONV);
			} elsif(($tmp) = /FracOnTime (\d+)/) {
				#print "p: $curP, q: $curQ, r: $curR, n: $curNode, val: $tmp\n";
				$fracOnTimeVals{$curP}{$curQ}
					{$curR}{$curNode} = ($tmp / $ON_TIME_BASE);
			}
		}
		close(STATS_OUT);
	}

	for($i = 0; $i < $numTests_; $i++) {
		for($j = 0 ; $j < scalar(@plotTypes_); $j++) {
			my $testPre = $testPre_.($i+1);
			my $datFname = $gnuplotDir.$testPre."_".
					$plotTypes_[$j].".dat";
			my $gpFname = $gnuplotDir.$testPre."_".
					$plotTypes_[$j].".gp";
			my $psFname = $gnuplotDir.$testPre."_".
					$plotTypes_[$j].".ps";
			my $epsFname = $gnuplotDir.$testPre."_".
					$plotTypes_[$j].".eps";

			open(DAT_OUT, '>', $datFname) || 
				die "Cannot open $datFname\n";
			open(GP_OUT, '>', $gpFname) || 
				die "Cannot open $gpFname\n";

			print GP_OUT "\n";
			if(0 && $doEps_ && $i == 0 && $plotTypes_[$j] eq "pct_recv") {
				print GP_OUT "set xrange [-0.2:1]\n";
			} else {
				print GP_OUT "set xrange [0:1]\n";
			}

			if(0 && $doEps_ && $i == 0 && $plotTypes_[$j] eq "latency") {
				print GP_OUT "set yrange [0:450]\n";
			} else {
				print GP_OUT "set yrange [0:]\n";
			}

			print GP_OUT "set xlabel \"$xLabels_[$i]\"\n";
			print GP_OUT "set ylabel \"".
				"$yLabels_{$plotTypes_[$j]}\"\n";

			if($doEps_ && $i == 0 && $plotTypes_[$j] eq "energy") {
				#print GP_OUT "set key outside\n";
				print GP_OUT "set key bottom left\n";
			} elsif($doEps_ && $i == 0 && $plotTypes_[$j] eq "latency") {
				print GP_OUT "set key bottom left\n";
			} elsif($doEps_ && $i == 0 && $plotTypes_[$j] eq "total_recv") {
				#print GP_OUT "set key 0.8,10\n";
				print GP_OUT "set key bottom left\n";
			} elsif($doEps_ && $i == 0 && $plotTypes_[$j] eq "total_sent") {
				print GP_OUT "set key bottom left\n";
			} elsif($doEps_ && $i == 1 && $plotTypes_[$j] eq "energy") {
				print GP_OUT "set key bottom right\n";
			} elsif($doEps_ && $i == 1 && $plotTypes_[$j] eq "latency") {
				print GP_OUT "set key 0.3, 110\n";
			} elsif($doEps_ && $i == 1 && $plotTypes_[$j] eq "pct_recv") {
				print GP_OUT "set key bottom right\n";
			} elsif($doEps_ && $i == 1 && $plotTypes_[$j] eq "total_recv") {
				print GP_OUT "set key bottom right\n";
			} elsif($doEps_ && $i == 2 && $plotTypes_[$j] eq "latency") {
				print GP_OUT "set key 0.8,100\n";
			} elsif($doEps_ && $i == 2 && $plotTypes_[$j] eq "pct_recv") {
				print GP_OUT "set key bottom right\n";
			} elsif($doEps_ && $i == 2 && $plotTypes_[$j] eq "total_recv") {
				print GP_OUT "set key bottom right\n";
			} elsif($doEps_ && $i == 2 && $plotTypes_[$j] eq "total_sent") {
				print GP_OUT "set key bottom right\n";
			} else {
				print GP_OUT "set key\n";
			}

			if($doEps_ == 1) {
				print GP_OUT "set size 0.6,0.6\n";
				print GP_OUT "set term postscript eps\n";
				print GP_OUT "set output \"$epsFname\"\n";
			} else {
				print GP_OUT "set term postscript\n";
				print GP_OUT "set output \"$psFname\"\n";
			}

			my %hashVals;

			if($plotTypes_[$j] eq "energy") {
				%hashVals = %fracOnTimeVals;
			} elsif ($plotTypes_[$j] eq "latency") {
				%hashVals = %avgLatVals;
			} elsif ($plotTypes_[$j] eq "pct_recv") {
				%hashVals = %totalAppRecvVals;
			} elsif ($plotTypes_[$j] eq "total_recv") {
				%hashVals = %totalDataRecvVals;
			} elsif ($plotTypes_[$j] eq "total_sent") {
				%hashVals = %totalDataSentVals;
			} else {
				die "Unknown plot type: $plotTypes_[$j]\n";
			}

			print GP_OUT "\n";

			my $isFirst = 1;
			my $printPlot = 1;
			my $curCol = 2;
			my $terminator = "";
			my %curVal;
			foreach (@{$protVals_{$xKey_[$i]}[$i]}) {
				print DAT_OUT ($_ / $NUM_PROB_LEVELS);
				$curVal{$xKey_[$i]} = $_;
				foreach (@{$protVals_{$param1Key_[$i]}[$i]}) {
					$curVal{$param1Key_[$i]} = $_;
					foreach (@{$protVals_{$param2Key_[$i]}[$i]}) {
						$curVal{$param2Key_[$i]} = $_;
						#print "pVal: $curVal{$pKey_}, ".
						#	"qVal: $curVal{$qKey_}, ".
						#	"rVal: $curVal{$rKey_} ".
						#	"is ".$hashVals{$curVal{$pKey_}}
						#	{$curVal{$qKey_}}{$curVal{$rKey_}}{1}."\n";
						print DAT_OUT " ".
							avgHash(\%{$hashVals{$curVal{$pKey_}}
							{$curVal{$qKey_}}{$curVal{$rKey_}}});
						if($printPlot) {
							print GP_OUT "plot \\\n";
							$terminator = "";
							$printPlot = 0;
						}

						if($isFirst) {
							print GP_OUT $terminator;
							print GP_OUT "\t\"$datFname\" ".
								"using 1:$curCol title ".
								"\'$param1Key_[$i]=".
								($curVal{$param1Key_[$i]}/10).", ".
								" $param2Key_[$i]=".
								($curVal{$param2Key_[$i]}/10)."\' ".
								"w lp";
							$terminator = ", \\\n";
							$curCol++;
						}
					}
				}
				$isFirst = 0;
				print DAT_OUT "\n";
			}

			print GP_OUT "\n\n";
			print DAT_OUT "\n";

			close(DAT_OUT);
			close(GP_OUT);

			system("gnuplot", $gpFname);

		} # end plotTypes_
	} # end numTests_

}

#### SUBROUTINES ####

sub avgHash {
	my $sum = 0.0;
	my $key;
	my $h_ptr = shift;
	my %h = %$h_ptr;
	my $size = scalar(keys(%h));
	foreach $key (keys(%h)) {
		$sum += $h{$key};	
	}
	return ($size > 0) ? ($sum/$size) : $sum;
}

#####
# Function from http://www.devdaily.com/perl/edu/
# articles/pl010005/prompt.txt
#
# Usage:
# $retVal = &promptUser("Prompt text ");
# OR
# $homeDir  = &promptUser("Prompt text ", "default value");
#####
sub promptUser {
	
	my ($promptStr,$defaultStr) = @_;

	if ($defaultStr) {
	   print $promptStr, "[", $defaultStr, "]: ";
	} else {
	   print $promptStr, ": ";
	}

	# force a flush after our print
	$| = 1;
	# get input
	$_ = <STDIN>;

	# remove extra spaces
	chomp;

	if ("$defaultStr") {
	   return $_ ? $_ : $defaultStr;
	} else {
	   return $_;
	}
}

