#!/usr/bin/perl -w
use pbbfGlobal;
use POSIX;
use strict;

my $doDebug_ = 0;

my @pVals_ = qw(0 3 7 10);
my @qVals_ = qw(0 3 7 10);
my @rVals_ = qw(0 3 7 10);

if($doDebug_) {
	@pVals_ = qw(0);
	@qVals_ = qw(0 1);
	@rVals_ = qw(0);
}

sub main;
sub runTests;
sub promptUser;

sub avg;
sub std_dev;
sub first_idx (&@);
sub get_stats;
sub plot_test;
sub plot_el;

main();

sub main {

	print "\n";

	my $promptVal = &promptUser("Are you sure that you want to ".
			"run tests (y/n)? ", "n");

	if($promptVal eq "y" || $promptVal eq "Y") {
		$promptVal = &promptUser("Is serial forwarder running? ".
				"(y/n)? ", "n");
		if($promptVal eq "y" || $promptVal eq "Y") {
			runTests();
		}
	}

	print "\nFinished!!!\n\n";
}

sub runTests {
	my ($i, $j, $k);
	my ($curP, $curQ, $curR);
	# We have to start the seqno at 1
	my $seqno = 1;
	my $statsDir = $pbbfGlobal::STATS_DIR;
	my $numRuns = 
		(scalar(@pVals_) * scalar(@qVals_) * scalar(@rVals_));
	my $fName;
	my $fNamePre = "PbbfComm";
	my $pbbfCommCmd = "java net.tinyos.simple_pbbf.PbbfComm";
	my @oldFiles;

	# Create a stats directory if it doesn't exist
	if(!(-d $statsDir)) {
		mkdir($statsDir, 0755) || die 
			"Cannot make directory $statsDir: $!";
	}

	# Remove any old files
	@oldFiles = glob($statsDir.$fNamePre."*");
	foreach $i (@oldFiles) {
		unlink $i;
	}

	print "\n";

	for($i = 0; $i < scalar(@pVals_); $i++) {
		for($j = 0; $j < scalar(@qVals_); $j++) {
			for($k = 0; $k < scalar(@rVals_); $k++) {
				$curP = $pVals_[$i];
				$curQ = $qVals_[$j];
				$curR = $rVals_[$k];

				print "Running seqno $seqno/$numRuns ".
					"(p=$curP,q=$curQ,r=$curR)\n";

				$fName = $statsDir.$fNamePre."-p$curP".
					"-q$curQ-r$curR-seqno$seqno.stats";
				system("$pbbfCommCmd -pVal $curP -qVal $curQ ".
					"-rVal $curR -seqno $seqno -file $fName ".
					">& /dev/null") || 
					die "system failed for p=$curP, ".
					"q=$curQ, r=$curR\n\n";
				$seqno++;
			} # k loop
		} # j loop
	} # i loop
}

#####
# Function from http://www.devdaily.com/perl/edu/
# articles/pl010005/prompt.txt
#
# Usage:
# $retVal = &promptUser("Prompt text ");
# OR
# $homeDir  = &promptUser("Prompt text ", "default value");
#####
sub promptUser {
	
	my ($promptStr,$defaultStr) = @_;

	if ($defaultStr) {
	   print $promptStr, "[", $defaultStr, "]: ";
	} else {
	   print $promptStr, ": ";
	}

	# force a flush after our print
	$| = 1;
	# get input
	$_ = <STDIN>;

	# remove extra spaces
	chomp;

	if ("$defaultStr") {
	   return $_ ? $_ : $defaultStr;
	} else {
	   return $_;
	}
}


