#!/usr/bin/perl -w
use POSIX;
use strict;

my $do_run = 0;
my $do_plot = 1;
my $do_eps = 0;
my $do_pres = 1;
my $do_ns = 0;
my $do_std_dev = 0;

# Choose which tests we want to run -- 1 == run, 0 == don't run
my @do_test = qw(0 0 1);
my @with_nacks = qw(0 0 0);
my @do_hop_lat = qw(0 0 0);
my @do_hop_hops = qw(0 0 1);
my @do_el = qw(0 0 0);
my @do_thresh = qw(0 0 0);
my @do_grid = qw(0 0 1);
# without nacks = "nack", with nacks = "nonack"
my $NUM_RUNS = ($do_ns == 1) ? 10 : 1;

# Test 1 = vary updates per packet (k)
# Test 2 = vary node density (d)
# Test 3 = vary q

# x parameters for tests
# Have to use qw for decimals.  Otherwise, 1.0 will look like
# 1 for hash key lookups and will fail
my @x_param;
$x_param[0] = [1 .. 5]; # k
#$x_param[0] = [1, 3]; # k
$x_param[1] = [qw(7.5 10.0 12.5 15.0 17.5)]; # node density
#$x_param[1] = [qw(10.0 15.0)]; # node density
$x_param[2] = [qw(0.0 0.125 0.25 0.375 0.5 0.625 0.75 0.875 0.95 0.99 1.0)]; # q
#$x_param[2] = [qw(0.0 0.083 0.167 0.25 0.5 0.75 1.0)]; # q

my @x_labels;
$x_labels[0] = "Number of Updates in Each Broadcast Packet";
#$x_labels[1] = "Node Density (Expected Number of One-Hop Neighbors)";
$x_labels[1] = "DELTA";
$x_labels[2] = "q";

my @prot;
$prot[0] = [qw(no_psm psm p_0.05 p_0.1 p_0.25 p_0.5)];
#$prot[0] = [qw(no_psm psm p_0.05)];
$prot[1] = [qw(no_psm psm p_0.05 p_0.1 p_0.25)];
#$prot[1] = [qw(no_psm psm p_0.05)];
$prot[2] = ($do_ns == 1) ?  [qw(no_psm psm p_0.05 p_0.1 p_0.25 p_0.5)] :
[qw(no_psm psm p_0.05 p_0.25 p_0.375 p_0.5 p_0.75)];
#$prot[2] = [qw(no_psm psm p_0.05)];

my %prot_labels;
$prot_labels{"no_psm"} = "NO PSM";
$prot_labels{"psm"} = "PSM";
$prot_labels{"p_0.05"} = "PBBF-0.05";
$prot_labels{"p_0.1"} = "PBBF-0.1";
$prot_labels{"p_0.25"} = "PBBF-0.25";
$prot_labels{"p_0.375"} = "PBBF-0.375";
$prot_labels{"p_0.5"} = "PBBF-0.5";
$prot_labels{"p_0.75"} = "PBBF-0.75";

# gnuplot line types
my $err_lt = 2;
my %prot_lt;
$prot_lt{"no_psm"} = 1;
$prot_lt{"psm"} = 3;
$prot_lt{"p_0.05"} = 4;
$prot_lt{"p_0.1"} = 5;
$prot_lt{"p_0.25"} = 6;
$prot_lt{"p_0.375"} = 9;
$prot_lt{"p_0.5"} = 7;
$prot_lt{"p_0.75"} = 8;

my %prot_pt;
$prot_pt{"no_psm"} = 1;
$prot_pt{"psm"} = 2;
$prot_pt{"p_0.05"} = 3;
$prot_pt{"p_0.1"} = 4;
$prot_pt{"p_0.25"} = 5;
$prot_pt{"p_0.375"} = 8;
$prot_pt{"p_0.5"} = 6;
$prot_pt{"p_0.75"} = 7;

my @plot_types = qw(energy latency pct_recv all_recv nacks_sent bcasts_sent 
	pkts_sent pkts_recv bytes_sent bytes_recv zero_updates recv_80
	recv_85 recv_90 recv_95 recv_99 recv_10 latency_perhop hops hops_perhop);

my %y_labels;
$y_labels{"energy"} = "Joules Consumed/\\nTotal Updates Sent at Source"; 
$y_labels{"latency"} = "Average Update Latency (s)"; 
$y_labels{"latency_perhop"} = "Average Per-Hop Update Latency (s)"; 
$y_labels{"hops"} = "Average Update Flooding Hop Count"; 
$y_labels{"hops_perhop"} = "Average Per-Hop Update\\nFlooding Hop Count"; 
$y_labels{"pct_recv"} = "Updates Received/\\nTotal ".
		"Updates Sent at Source";
$y_labels{"all_recv"} = "Fraction of Updates Recieved\\nBy All Nodes";
$y_labels{"recv_80"} = "Fraction of Updates Recieved\\nBy 80% Nodes";
$y_labels{"recv_85"} = "Fraction of Updates Recieved\\nBy 85% Nodes";
$y_labels{"recv_90"} = "Fraction of Updates Recieved\\nBy 90% Nodes";
$y_labels{"recv_95"} = "Fraction of Updates Recieved\\nBy 95% Nodes";
$y_labels{"recv_99"} = "Fraction of Updates Recieved\\nBy 99% Nodes";
$y_labels{"recv_10"} = "Fraction of Updates Recieved\\nBy 10% Nodes";
$y_labels{"nacks_sent"} = "NACKs Sent/Total ".
		"Updates Sent at Source";
$y_labels{"bcasts_sent"} = "Broadcasts Sent/Total ".
		"Updates Sent at Source";
$y_labels{"pkts_sent"} = "Packets Sent/Total ".
		"Updates Sent at Source";
$y_labels{"pkts_recv"} = "Packets Received/Total ".
		"Updates Sent at Source";
$y_labels{"bytes_sent"} = "Bytes Sent/Total ".
		"Updates Sent at Source";
$y_labels{"bytes_recv"} = "Bytes Received/Total ".
		"Updates Sent at Source";
$y_labels{"zero_updates"} = "Percent of Nodes Receiving Zero Updates";

sub avg;
sub std_dev;
sub first_idx (&@);
sub run_test;
sub get_stats;
sub plot_test;
sub plot_el;

sub avg_wo_undef;
sub std_dev_wo_undef;

srand(2003);


my $TRACE_DIR = "trace_out/";
my $STATS_DIR = "stats_out/";
my $SCEN_DIR = "scen_out/";
my $GNUPLOT_DIR = "gnuplot_out/";

my $PI = 3.14159265;

# ns parameters
my $num_nodes = ($do_ns == 1) ? 50 : 5625; 
my $sim_time = ($do_ns == 1) ? 500: 10000; # seconds
my $range = 250; # meters
my $rate = "0.01"; #"0.2"; # pkts/sec
my $file_size_avg = "30.0"; #"128.0"; # Bytes
my $file_size_dev = "0.0"; #"64.0"; # Bytes
my $file_size_min = "32.0"; # Bytes
my $file_size_max = "256.0"; # Bytes
my $fixed_q = "0.25"; # pr node stays on if not recving ATIM or ACK
my $fixed_k = 1; # num updates per packet
my $fixed_d = "10.0"; # expected scenario density

my $args = join(" ",@ARGV);
my $just_stats = 0;
if($args =~ /.*-get_stats(?:$|\s+.*)/) {
	my ($stats_fname) = ($args =~ /.*-s\s+(\S+)(?:$|\s+.*).*/);
	my ($trace_fname) = ($args =~ /.*-t\s+(\S+)(?:$|\s+.*).*/);
	if(!(defined $stats_fname && defined $trace_fname)) { 
		die "File names no specified correctly\n\n";
	}
	get_stats($trace_fname, $stats_fname);
	die "\n";
}

# generate all seeds so we don't get different results
# with different tests being run
my ($i, @all_seeds);
for($i = 0; $i < (@do_test * $NUM_RUNS); $i++) {
	$all_seeds[$i] = floor(rand 100000);
}
for($i = 0; $i < @do_test; $i++) {
	if($do_test[$i] == 1) {
		if($do_run == 1) { run_test($i); }
		if($do_plot == 1) { plot_test($i); }
	}
}

print "\nFinished!!!\n\n";

#### SUBROUTINES ####

sub avg_wo_undef {
	my @tmp;
	foreach $i (@_) {
		if(defined $i) {
			push(@tmp, $i);
		}
	}
	return (avg(@tmp));
}

sub std_dev_wo_undef {
	my @tmp;
	foreach $i (@_) {
		if(defined $i) {
			push(@tmp, $i);
		}
	}
	return (std_dev(@tmp));
}

sub avg {
	my $sum = 0.0;
	foreach $i (@_) {
		$sum += $i;	
	}
	return (@_ > 0) ? ($sum/@_) : $sum;
}

sub std_dev {
	my $avg = avg(@_);
	my $sum = 0.0;
	foreach $i (@_) {
		my $tmp = ($i - $avg); 
		$sum += ($tmp * $tmp);	
	}
	return (@_ > 1) ? (sqrt($sum/(@_ - 1))) : 0.0;
}

sub first_idx (&@) {
	my $test = shift;
	my $idx = 0;
	local $_;
	foreach (@_) {
		return $idx if $test->();
		$idx++;
	}
	return undef;
}

sub run_test {

	my $test_idx = $_[0];
	my $test_num = $test_idx + 1;
	#my $test_pre = "code_app".$test_num;
	my $test_pre = "code_app".$test_num;
	$test_pre = ($with_nacks[$test_idx] == 1) ? 
		$test_pre."_nack" : $test_pre."_nonack";
	$test_pre = ($do_grid[$test_idx] == 1) ? 
		$test_pre."_grid" : $test_pre."_rand";
	$test_pre = ($do_ns == 1) ? 
		$test_pre."_ns" : $test_pre."_perl";
	local *STATS_OUT;

	my ($i, $j, $k);
	my $cur_num = 1;

	# Unlink any stat, gp, or ps files that may exist from this 
	# test previously
	my @prev_fnames = glob($STATS_DIR.$test_pre."*");
	push(@prev_fnames,glob($SCEN_DIR.$test_pre."*"));
	push(@prev_fnames,glob($TRACE_DIR.$test_pre."*"));
	foreach $i (@prev_fnames) {
		unlink $i;
	}

	# Generate random seeds
	my @seeds;
	for($i = 0; $i < $NUM_RUNS; $i++) {
		$seeds[$i] = $all_seeds[(($NUM_RUNS * $test_idx) + $i)];
	}

	my $total_runs = (@{$x_param[$test_idx]} * @seeds * @{$prot[$test_idx]});

	for($i = 0; $i < @{$x_param[$test_idx]}; $i++) {
		my $cur_k = ($test_idx == 0) ? $x_param[$test_idx][$i] : $fixed_k;
		my $cur_d = ($test_idx == 1) ? $x_param[$test_idx][$i] : $fixed_d;
		my $cur_q = ($test_idx == 2) ? $x_param[$test_idx][$i] : $fixed_q;
		for($j = 0; $j < @seeds; $j++) {
			my $cur_run = $j+1;
			my $cur_seed = $seeds[$j];
			my $scen_fname = $SCEN_DIR.$test_pre."-nn$num_nodes".
				"-d$cur_d-rs$cur_seed.scen";

			# calculate the area based on the desired density
			my $area = ($PI * $num_nodes * ($range**2)) / $cur_d;
			my $x_val = ($do_grid[$test_idx] == 1) ? 
				((floor(sqrt($num_nodes))-1) * $range) : sqrt($area);
			my $y_val = $x_val;
			my $cust_p_str = ($do_grid[$test_idx] == 1) ? 1 : 0;

			if($do_ns == 1) {
			system("perl mkscen.pl -nn $num_nodes -x $x_val -y $y_val ".
				"-f $scen_fname -cust 1 -cust_p $cust_p_str ".
				"-rs $cur_seed >& /dev/null");
			}

		for($k = 0; $k < @{$prot[$test_idx]}; $k++) {

			my $cur_protocol = $prot[$test_idx][$k];
	
			my $trace_fname = $TRACE_DIR.$test_pre."-nn$num_nodes-".
				"k$cur_k-d$cur_d-q$cur_q-$cur_protocol-run$cur_run-".
				"seed$cur_seed.tr";

			print "Running code_app_tests/test".$test_num.
					": $cur_num/$total_runs\n";
			$cur_num++;

			my $psm_val = "OFF";
			my $p_val = 0.0;
			my $q_val = 0.0;
			my $nack_val = ($with_nacks[$test_idx] == 1) ? "ON" : "OFF";
			my $tmp;

			if($cur_protocol eq "no_psm") {
				$psm_val = "OFF";
			}
			elsif($cur_protocol eq "psm") {
				$psm_val = "ON";
				$p_val = 0.0;
				$q_val = 0.0;
			}
			elsif(($tmp) = ($cur_protocol =~ /p_(\d+\.\d+)/ )) {
				$psm_val = "ON";
				$p_val = $tmp;
				$q_val = $cur_q;
			}
			else {
				print "UNKNOWN PROTOCOL!!!\n";
			}

			if($do_ns == 1) {
			system("../ns run-test.tcl -sc $scen_fname -nn $num_nodes -x $x_val ".
				"-y $y_val -psm $psm_val -send_nack_opt $nack_val ".
				"-q_val $q_val -p_val $p_val -num_updates $cur_k ".
				"-filesize_avg $file_size_avg ".
				"-filesize_dev $file_size_dev -filesize_max $file_size_max ".
				"-filesize_min $file_size_min -stop $sim_time ".
				"-sending_rate $rate ".
				"-f_out $trace_fname -seed $cur_seed >& /dev/null");
			} else {
				system("perl pq_sim.pl -f_out $trace_fname -nn $num_nodes ".
					"-num_msgs 100 -psm $psm_val -p_val $p_val ".
					"-q_val $q_val -rs $cur_seed >& /dev/null");
			}

			# Add (append) averages to a stats file
			my $stats_fname = $STATS_DIR.$test_pre.
				"-k$cur_k-d$cur_d-q$cur_q-$cur_protocol.stats";
			open(STATS_OUT,'>>', $stats_fname) || die "Cannot open $stats_fname\n";
			print STATS_OUT "\nPROTOCOL: $cur_protocol, RUN: $cur_run, ".
				"SEED: $cur_seed\n";
			close(STATS_OUT);
			get_stats($trace_fname, $stats_fname);
			unlink($trace_fname);
			
		} # end protocols
		} # end num runs
	} # end x_param

	print "\n";

}

sub get_stats {

	my $in_fname = shift;
	my $out_fname = shift;
	local (*IN, *OUT);

	open(IN, $in_fname) || die "Cannot open $in_fname\n";
	open(OUT, '>>', $out_fname) || die "Cannot open $out_fname\n";

	my $code_src = undef;
	my %num_updates_hash;
	my %dist_from_src;
	my $dist = undef;
	my $key = undef;
	my $zero_updates = 0;
	my @update_num_recv;
	my $i;

	while(<IN>) {
		(my $event) = /^(\w)\s+/;
		if($event ne "l") { next; }

		(my $node_id) = /.*-Ni\s+(\d+)\s+/;

		(my $dest) = /.*-dest\s+(\d+)\s+/;
		($dist) = /.*-dist\s+(\d+)\s+/;

		if((defined $dest) && (defined $dist)) {
			$dist_from_src{$dest} = $dist;
		}

		(my $num_updates) = /.*-numUpdates\s+(\d+)(?:$|\s+.*)/;
		if(defined $num_updates) {
			$num_updates_hash{$node_id} = $num_updates;
			if($num_updates == 0) {
				$zero_updates++;
			}
		}

		if(/.*-isCodeSource(?:$|\s+.*)/) {
			$code_src = $node_id;
		}
	}

	if(defined $code_src && defined $num_updates_hash{$code_src} && 
			$num_updates_hash{$code_src} > 0) {

		my $total_updates = $num_updates_hash{$code_src};
		my $sum_energy_used = 0.0;
		my $sum_pct_updates = 0.0;
		my $sum_nacks_sent = 0;
		my $sum_bcasts_sent = 0;
		my $sum_pkts_sent = 0;
		my $sum_pkts_recv = 0;
		my $sum_bytes_sent = 0;
		my $sum_bytes_recv = 0;
		my $sum_avg_update_lat = 0;
		my $sum_avg_update_lat_perhop = 0;
		my $sum_avg_update_hops = 0;
		my $sum_avg_update_hops_perhop = 0;
		my $all_recv = 0;
		my $recv_80 = 0;
		my $recv_85 = 0;
		my $recv_90 = 0;
		my $recv_95 = 0;
		my $recv_99 = 0;
		my $recv_10 = 0;
		my %sum_hop_lat;
		my %num_hop_lat;
		my %sum_hop_hops;
		my %num_hop_hops;

		seek(IN, 0, 0);

		# Keep a count of how many nodes received a given update
		# This tells us whether all nodes received the update or not
		for($i = 0; $i < $total_updates; $i++) {
			$update_num_recv[$i] = 0;
		}
	
		while(<IN>) {
	
			(my $event) = /^(\w)\s+/;
			if($event ne "l") { next; }
	
			(my $node_id) = /.*-Ni\s+(\d+)\s+/;

			(my $energy_used) = /.*-energyUsed\s+(\d+\.\d+)(?:$|\s+.*)/;
			(my $num_updates) = /.*-numUpdates\s+(\d+)(?:$|\s+.*)/;
			(my $num_nacks_sent) = /.*-numNacksSent\s+(\d+)(?:$|\s+.*)/;
			(my $num_bcasts_sent) = /.*-numBroadcastsSent\s+(\d+)(?:$|\s+.*)/;
			(my $num_pkts_sent) = /.*-numPacketsSent\s+(\d+)(?:$|\s+.*)/;
			(my $num_pkts_recv) = /.*-numPacketsRecv\s+(\d+)(?:$|\s+.*)/;
			(my $num_bytes_sent) = /.*-numBytesSent\s+(\d+)(?:$|\s+.*)/;
			(my $num_bytes_recv) = /.*-numBytesRecv\s+(\d+)(?:$|\s+.*)/;
			(my $avg_update_lat) = /.*-avgUpdateLatency\s+(\d+\.\d+)(?:$|\s+.*)/;
			(my $avg_update_hops) = /.*-avgUpdateHops\s+(\d+\.\d+)(?:$|\s+.*)/;
			(my $update_num) = /.*-updateNum\s+(\d+)(?:$|\s+.*)/;

			if(defined $num_updates && $node_id != $code_src) {
				$sum_pct_updates += ($num_updates/$total_updates);
			}
			elsif(defined $update_num) {
				$update_num_recv[$update_num]++;
			}
			elsif(defined $energy_used && defined $num_updates_hash{$node_id} 
					&& $num_updates_hash{$node_id} > 0 && $node_id != $code_src) {
				$sum_energy_used += ($energy_used/$total_updates);
			}
			elsif(defined $num_nacks_sent && defined $num_updates_hash{$node_id} 
					&& $num_updates_hash{$node_id} > 0 && $node_id != $code_src) {
				$sum_nacks_sent += ($num_nacks_sent/$total_updates);
			}
			elsif(defined $num_bcasts_sent && defined $num_updates_hash{$node_id} 
					&& $num_updates_hash{$node_id} > 0 && $node_id != $code_src) {
				$sum_bcasts_sent += ($num_bcasts_sent/$total_updates);
			}
			elsif(defined $num_pkts_sent && defined $num_updates_hash{$node_id} 
					&& $num_updates_hash{$node_id} > 0 && $node_id != $code_src) {
				$sum_pkts_sent += ($num_pkts_sent/$total_updates);
			}
			elsif(defined $num_pkts_recv && defined $num_updates_hash{$node_id} 
					&& $num_updates_hash{$node_id} > 0 && $node_id != $code_src) {
				$sum_pkts_recv += ($num_pkts_recv/$total_updates);
			}
			elsif(defined $num_bytes_sent && defined $num_updates_hash{$node_id} 
					&& $num_updates_hash{$node_id} > 0 && $node_id != $code_src) {
				$sum_bytes_sent += ($num_bytes_sent/$total_updates);
			}
			elsif(defined $num_bytes_recv && defined $num_updates_hash{$node_id} 
					&& $num_updates_hash{$node_id} > 0 && $node_id != $code_src) {
				$sum_bytes_recv += ($num_bytes_recv/$total_updates);
			}
			elsif(defined $avg_update_lat && defined $num_updates_hash{$node_id} 
					&& $num_updates_hash{$node_id} > 0 && $node_id != $code_src) {
				$dist = $dist_from_src{$node_id};
				if(defined $dist && $dist > 0) {
					$sum_avg_update_lat_perhop += ($avg_update_lat/$dist);
					$sum_avg_update_lat += $avg_update_lat;
					if(!(defined $num_hop_lat{$dist})) {
						$num_hop_lat{$dist} = 1;	
						$sum_hop_lat{$dist} = $avg_update_lat;	
					} else {
						$num_hop_lat{$dist}++;	
						$sum_hop_lat{$dist} += $avg_update_lat;
					}
				}
			}
			elsif(defined $avg_update_hops && defined $num_updates_hash{$node_id} 
					&& $num_updates_hash{$node_id} > 0 && $node_id != $code_src) {
				$dist = $dist_from_src{$node_id};
				if(defined $dist && $dist > 0) {
					$sum_avg_update_hops_perhop += ($avg_update_hops/$dist);
					$sum_avg_update_hops += $avg_update_hops;
					if(!(defined $num_hop_hops{$dist})) {
						$num_hop_hops{$dist} = 1;	
						$sum_hop_hops{$dist} = $avg_update_hops;	
					} else {
						$num_hop_hops{$dist}++;	
						$sum_hop_hops{$dist} += $avg_update_hops;
					}
				}
			}
		}

		for($i = 0; $i < $total_updates; $i++) {
			if($update_num_recv[$i] == $num_nodes) {
				$all_recv++;
			}
			if($update_num_recv[$i] >= (0.8 * $num_nodes)) {
				$recv_80++;
			}
			if($update_num_recv[$i] >= (0.85 * $num_nodes)) {
				$recv_85++;
			}
			if($update_num_recv[$i] >= (0.9 * $num_nodes)) {
				$recv_90++;
			}
			if($update_num_recv[$i] >= (0.95 * $num_nodes)) {
				$recv_95++;
			}
			if($update_num_recv[$i] >= (0.99 * $num_nodes)) {
				$recv_99++;
			}
			if($update_num_recv[$i] >= (0.1 * $num_nodes)) {
				$recv_10++;
			}
		}

		if($zero_updates < ($num_nodes - 1)) {
			print OUT "avg joules per bit: ".
					sprintf("%.9f",
					($sum_energy_used/($num_nodes - 1 - $zero_updates)))."\n";
			print OUT "update pct: ".
					sprintf("%.9f",
					($sum_pct_updates/($num_nodes - 1 - $zero_updates)))."\n";
			print OUT "all recv pct: ".
					sprintf("%.9f",
					($all_recv/$total_updates))."\n";
			print OUT "recv 80 pct: ".
					sprintf("%.9f",
					($recv_80/$total_updates))."\n";
			print OUT "recv 85 pct: ".
					sprintf("%.9f",
					($recv_85/$total_updates))."\n";
			print OUT "recv 90 pct: ".
					sprintf("%.9f",
					($recv_90/$total_updates))."\n";
			print OUT "recv 95 pct: ".
					sprintf("%.9f",
					($recv_95/$total_updates))."\n";
			print OUT "recv 99 pct: ".
					sprintf("%.9f",
					($recv_99/$total_updates))."\n";
			print OUT "recv 10 pct: ".
					sprintf("%.9f",
					($recv_10/$total_updates))."\n";
			print OUT "nacks sent: ".
					sprintf("%.9f",
					($sum_nacks_sent/($num_nodes - 1 - $zero_updates)))."\n";
			print OUT "bcasts sent: ".
					sprintf("%.9f",
					($sum_bcasts_sent/($num_nodes - 1 - $zero_updates)))."\n";
			print OUT "pkts sent: ".
					sprintf("%.9f",
					($sum_pkts_sent/($num_nodes - 1 - $zero_updates)))."\n";
			print OUT "pkts recv: ".
					sprintf("%.9f",
					($sum_pkts_recv/($num_nodes - 1 - $zero_updates)))."\n";
			print OUT "bytes sent: ".
					sprintf("%.9f",
					($sum_bytes_sent/($num_nodes - 1 - $zero_updates)))."\n";
			print OUT "bytes recv: ".
					sprintf("%.9f",
					($sum_bytes_recv/($num_nodes - 1 - $zero_updates)))."\n";
			print OUT "avg update latency: ".
					sprintf("%.9f",
					($sum_avg_update_lat/($num_nodes - 1 - $zero_updates)))."\n";
			print OUT "avg update latency perhop: ".
					sprintf("%.9f",
					($sum_avg_update_lat_perhop/($num_nodes - 1 - $zero_updates)))."\n";
			print OUT "avg update hops: ".
					sprintf("%.9f",
					($sum_avg_update_hops/($num_nodes - 1 - $zero_updates)))."\n";
			print OUT "avg update hops perhop: ".
					sprintf("%.9f",
					($sum_avg_update_hops_perhop/($num_nodes - 1 - $zero_updates)))."\n";

			foreach $key (keys(%num_hop_lat)) {
				print OUT "avg $key-hop update latency: ".
						sprintf("%.9f", ($sum_hop_lat{$key}/$num_hop_lat{$key})).
						" (N=$num_hop_lat{$key})\n";
			}

			foreach $key (keys(%num_hop_hops)) {
				print OUT "avg $key-hop update hops: ".
						sprintf("%.9f", ($sum_hop_hops{$key}/$num_hop_hops{$key})).
						" (N=$num_hop_hops{$key})\n";
			}

		}

		if($num_nodes > 1) {
			print OUT "zero updates: ".
					sprintf("%.9f",
					($zero_updates/($num_nodes - 1)))."\n";
		}

	}

	print OUT "\n";

	close(IN);
	close(OUT);
}

sub plot_test {

	my $test_idx = $_[0];
	my $test_num = $test_idx + 1;
	#my $test_pre = "code_app".$test_num;
	my $test_pre = "code_app".$test_num;
	$test_pre = ($with_nacks[$test_idx] == 1) ? 
		$test_pre."_nack" : $test_pre."_nonack";
	$test_pre = ($do_grid[$test_idx] == 1) ? 
		$test_pre."_grid" : $test_pre."_rand";
	$test_pre = ($do_ns == 1) ? 
		$test_pre."_ns" : $test_pre."_perl";

	my ($i,$j,$key,$tmp,$dist,$num,$str,%titles,$line_title);
	my (%e_vals, %lat_vals, %update_pct_vals, 
			%all_recv_vals, %nacks_sent_vals, 
			%recv_80_vals, %recv_85_vals, %recv_90_vals, %recv_95_vals,
			%recv_99_vals, %recv_10_vals, %lat_perhop_vals,
			%bcasts_sent_vals, %pkts_sent_vals, %pkts_recv_vals,
			%bytes_sent_vals, %bytes_recv_vals, %zero_updates_vals,
			%hop_lat_vals, %sum_hop_num_scens, %total_hop_num_scens,
			%hops_vals, %hops_perhop_vals, %hop_hops_vals, %grid_hops);
	my ($cur_prot);
	my (@old_plot_types, %old_y_labels);
	my ($dat_fname, $gp_fname, $ps_fname, $eps_fname);
	local (*STATS_OUT);
	local (*DAT_OUT, *GP_OUT);

	my @prev_fnames = glob($GNUPLOT_DIR.$test_pre."*");
	foreach $i (@prev_fnames) {
		unlink $i;
	}

	# Go through all files with the appropiate prefix
	# and store the energy values in an array
	@prev_fnames = glob($STATS_DIR.$test_pre."*");
	foreach $i (@prev_fnames) {

		my $x_val;
		if($test_idx == 0) {
			($x_val) = ($i =~ /.*-k(\d+).*/);
		}
		elsif($test_idx == 1) {
			($x_val) = ($i =~ /.*-d(\d+\.\d+).*/);
		}
		elsif($test_idx == 2) {
			($x_val) = ($i =~ /.*-q(\d+\.\d+).*/);
		}
		else {
			die "Invalid test number in plot_test!!!\n";
		}

		for($j = 0; $j < @{$prot[$test_idx]}; $j++) {
			($cur_prot) = ($i =~ /.*-(\Q$prot[$test_idx][$j]\E).*/);
			if(defined $cur_prot) { last; }
		}

		$j = first_idx {$_ eq $x_val} @{$x_param[$test_idx]};

		open(STATS_OUT, $i) || die "Cannot open $i\n";
		my $cur_run = undef;
		while(<STATS_OUT>) {

			if(($tmp) = /^PROTOCOL:.*RUN: (\d+).*/) {
				$cur_run = $tmp;
			}
			elsif((($tmp) = /^update pct: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$update_pct_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^all recv pct: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$all_recv_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^recv 80 pct: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$recv_80_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^recv 85 pct: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$recv_85_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^recv 90 pct: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$recv_90_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^recv 95 pct: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$recv_95_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^recv 99 pct: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$recv_99_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^recv 10 pct: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$recv_10_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^avg update latency: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$lat_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^avg update latency perhop: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$lat_perhop_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($dist,$tmp,$num) = 
					/^avg (\d+)-hop update latency: (\d+\.\d+) \(N=(\d+)\).*/) && 
					defined $cur_run) {
				$hop_lat_vals{$dist}{$cur_prot}[$j][($cur_run-1)] = $tmp;
				if($cur_prot eq "no_psm") {
					$grid_hops{$dist} = $num;
				}
				# Keep track of average dist-hop neighbors per scenario
				if(!(defined $total_hop_num_scens{$dist})) {
					$total_hop_num_scens{$dist} = 1;
					$sum_hop_num_scens{$dist} = $num;
				} else {
					$total_hop_num_scens{$dist}++;
					$sum_hop_num_scens{$dist} += $num;
				}
			}
			elsif((($tmp) = /^avg update hops: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$hops_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^avg update hops perhop: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$hops_perhop_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($dist,$tmp,$num) = 
					/^avg (\d+)-hop update hops: (\d+\.\d+) \(N=(\d+)\).*/) && 
					defined $cur_run) {
				$hop_hops_vals{$dist}{$cur_prot}[$j][($cur_run-1)] = $tmp;
				# Average dist-hop neighbors per scenario is recorded
				# in the update latency loop
			}
			elsif((($tmp) = /^nacks sent: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$nacks_sent_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^bcasts sent: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$bcasts_sent_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^pkts sent: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$pkts_sent_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^pkts recv: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$pkts_recv_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^bytes sent: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$bytes_sent_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^bytes recv: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$bytes_recv_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^zero updates: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$zero_updates_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}
			elsif((($tmp) = /^avg joules per bit: (\d+\.\d+).*/) && 
					defined $cur_run) {
				$e_vals{$cur_prot}[$j][($cur_run-1)] = $tmp;
			}

		}

		close(STATS_OUT);

	}

	@old_plot_types = @plot_types;
	%old_y_labels = %y_labels;

	if($do_el[$test_idx] == 1) {
		my @sorted_keys;
		my %el_vals;
		my $el_pre = "el_99";
		$dat_fname = $GNUPLOT_DIR.$test_pre."_".$el_pre.".dat";
		$gp_fname = $GNUPLOT_DIR.$test_pre."_".$el_pre.".gp";
		$ps_fname = $GNUPLOT_DIR.$test_pre."_".$el_pre.".ps";
		$eps_fname = $GNUPLOT_DIR.$test_pre."_".$el_pre.".eps";
		foreach $cur_prot (@{$prot[$test_idx]}) {
			for($j = 0; $j < @{$x_param[$test_idx]}; $j++) {

				if(avg_wo_undef(@{$recv_99_vals{$cur_prot}[$j]}) == 1) {
					$el_vals{avg_wo_undef(@{$lat_perhop_vals{$cur_prot}[$j]})} =
						avg_wo_undef(@{$e_vals{$cur_prot}[$j]});
					last;
				}
			}
		}

		@sorted_keys = sort {$a <=> $b} keys(%el_vals);
		open(DAT_OUT, '>', $dat_fname) || die "Cannot open $dat_fname\n";
		open(GP_OUT, '>', $gp_fname) || die "Cannot open $gp_fname\n";
		print GP_OUT "\n";
		print GP_OUT "set xrange [0:]\n";
		print GP_OUT "set yrange [0:]\n";
		if($do_pres == 1) {
			print GP_OUT "set xlabel\n";
			print GP_OUT "set ylabel\n";
			print GP_OUT "set key off\n";
		} else {
			print GP_OUT "set xlabel \"Average Per-Hop Update Latency (s)\"\n";
			print GP_OUT "set ylabel \"Joules Consumed/\\nTotal Updates Sent ".
				"at Source\"\n";
			print GP_OUT "set key\n";
		}

		if($do_eps == 1) {
			print GP_OUT "set size 0.6,0.6\n";
			print GP_OUT "set term postscript eps\n";
			print GP_OUT "set output \"$eps_fname\"\n";
			#print GP_OUT "set title \"99%\"\n";
		}
		else {
			print GP_OUT "set term postscript\n";
			print GP_OUT "set output \"$ps_fname\"\n";
		}
		print GP_OUT "\n";
		print GP_OUT "plot \"$dat_fname\" using 1:2 notitle w lp\n\n";

		foreach $key (@sorted_keys) {
			print DAT_OUT "$key $el_vals{$key}\n";
		}
		system("gnuplot", $gp_fname);
	}

	if($do_thresh[$test_idx] == 1) {
		my (@sorted_keys, @sorted_thresh_vals, @dat_lines, %thresh_vals);
		my $thresh_pre = "thresh";
		my ($pct, $p_val);
		foreach $cur_prot (@{$prot[$test_idx]}) {
			# parse p val into $p_val
			if(!(($p_val) = ($cur_prot =~ /p_(\d+\.\d+)/))) { next; }

			for($j = 0; $j < @{$x_param[$test_idx]}; $j++) {

				if(avg_wo_undef(@{$recv_99_vals{$cur_prot}[$j]}) == 1 && 
						!(defined $thresh_vals{99}{$p_val})) {
					$thresh_vals{99}{$p_val} = 
						$p_val * (1.0 - $x_param[$test_idx][$j]);
				}

				if(avg_wo_undef(@{$recv_90_vals{$cur_prot}[$j]}) == 1 && 
						!(defined $thresh_vals{90}{$p_val})) {
					$thresh_vals{90}{$p_val} = 
						$p_val * (1.0 - $x_param[$test_idx][$j]);
				}

				if(avg_wo_undef(@{$recv_80_vals{$cur_prot}[$j]}) == 1 && 
						!(defined $thresh_vals{80}{$p_val})) {
					$thresh_vals{80}{$p_val} = 
						$p_val * (1.0 - $x_param[$test_idx][$j]);
				}
			}
		}


		$dat_fname = $GNUPLOT_DIR.$test_pre."_".$thresh_pre.".dat";
		$gp_fname = $GNUPLOT_DIR.$test_pre."_".$thresh_pre.".gp";
		$ps_fname = $GNUPLOT_DIR.$test_pre."_".$thresh_pre.".ps";
		$eps_fname = $GNUPLOT_DIR.$test_pre."_".$thresh_pre.".eps";

		open(DAT_OUT, '>', $dat_fname) || die "Cannot open $dat_fname\n";
		open(GP_OUT, '>', $gp_fname) || die "Cannot open $gp_fname\n";
		print GP_OUT "\n";
		print GP_OUT "set xrange [-0.05:1.05]\n";
		print GP_OUT "set yrange [0:1]\n";
		if($do_pres == 1) {
			print GP_OUT "set xlabel\n";
			print GP_OUT "set ylabel\n";
			print GP_OUT "set key off\n";
		} else {
			print GP_OUT "set xlabel \"p\"\n";
			print GP_OUT "set ylabel \"p(1-q)\nfor Specified Reliability\"\n";
			print GP_OUT "set key\n";
		}
		if($do_eps == 1) {
			print GP_OUT "set size 0.6,0.6\n";
			print GP_OUT "set term postscript eps\n";
			print GP_OUT "set output \"$eps_fname\"\n";
		}
		else {
			print GP_OUT "set term postscript\n";
			print GP_OUT "set output \"$ps_fname\"\n";
		}
		print GP_OUT "\n";

		print GP_OUT "plot \\\n";

		my ($j, $k);
		$j = 0;
		@sorted_thresh_vals = sort {$a <=> $b} keys(%thresh_vals);
		foreach $thresh_pre (@sorted_thresh_vals){

			$pct = $thresh_pre;
			@sorted_keys = sort {$a <=> $b} keys(%{$thresh_vals{$thresh_pre}});

			$line_title = "title \'$pct% Reliability\'";
			if($do_pres == 1) {
				$line_title = "notitle";
			}

			print GP_OUT "\"$dat_fname\" using ".(2*$j + 1).":".(2*$j + 2).
				" $line_title w lp";

			if($j == scalar(keys(%thresh_vals)) - 1) {
				print GP_OUT "\n\n";
			} else {
				print GP_OUT ", \\\n";
			}

			$k = 0;
			foreach $key (@sorted_keys) {
				# If this condition is met, then the specified reliability
				# is met when q=0 which we do not want to plot.  This holds
				# because p = p(1-q) when q=0.
				if($key == $thresh_vals{$thresh_pre}{$key}) { next; }

				$dat_lines[$k][$j] = "$key $thresh_vals{$thresh_pre}{$key}";
				$k++;
			}
			$j++;
		}

		for($k = 0; $k < @dat_lines; $k++) {
			for($j = 0; $j < scalar(keys(%thresh_vals));$j++) {
				if(defined $dat_lines[$k][$j]) {
					print DAT_OUT $dat_lines[$k][$j]." ";
				} else {
					print DAT_OUT "? ? ";
				}
			}
			print DAT_OUT "\n";
		}

		system("gnuplot", $gp_fname);

	}


	if($do_hop_lat[$test_idx] == 1) {
		foreach $key (keys(%hop_lat_vals)) {
			$str = "$key-hop_lat";
			$num = ($sum_hop_num_scens{$key} / $total_hop_num_scens{$key});
			push(@plot_types, $str);
			$y_labels{$str} = "Average $key-Hop Latency (s)";
			$titles{$str} = "Average Number of $key-Hop Nodes/Scenario = ".
					sprintf("%.4f", $num);
		}
	}

	if($do_hop_hops[$test_idx] == 1) {
		foreach $key (keys(%hop_hops_vals)) {

			if(($key % 10) != 0) { next; }

			$str = "$key-hop_hops";
			$num = ($sum_hop_num_scens{$key} / $total_hop_num_scens{$key});
			push(@plot_types, $str);
			$y_labels{$str} = "Average $key-Hop\\nFlooding Hop Count";
			$titles{$str} = "Number of $key-Hop Nodes in Grid = ".
					"$grid_hops{$key}";
			#		sprintf("%.4f", $grid_hops{$key});
		}
	}

	for($i = 0; $i < @plot_types; $i++) {

		$dat_fname = $GNUPLOT_DIR.$test_pre."_".$plot_types[$i].".dat";
		$gp_fname = $GNUPLOT_DIR.$test_pre."_".$plot_types[$i].".gp";
		$ps_fname = $GNUPLOT_DIR.$test_pre."_".$plot_types[$i].".ps";
		$eps_fname = $GNUPLOT_DIR.$test_pre."_".$plot_types[$i].".eps";

		# Create a GP data file with averages and std_dev's
		open(DAT_OUT, '>', $dat_fname) || die "Cannot open $dat_fname\n";
		# Create a GP script file to generate graphs
		open(GP_OUT, '>', $gp_fname) || die "Cannot open $gp_fname\n";

		# Set up script 
		print GP_OUT "\n";
		if($test_idx == 0) {
			print GP_OUT "set xrange [0:".
				($x_param[$test_idx][(@{$x_param[$test_idx]}-1)] +
				$x_param[$test_idx][0])."]\n";
		} elsif($test_idx == 1) {
			print GP_OUT "set xrange [7:18]\n";
		} elsif($test_idx == 2) {
			if($do_ns == 0 && $plot_types[$i] eq "recv_99") {
				print GP_OUT "set xrange [-0.18:1.05]\n";
			} else {
				print GP_OUT "set xrange [-0.05:".
					($x_param[$test_idx][(@{$x_param[$test_idx]}-1)] +
					0.05)."]\n";
			}
		} else {
			die "Unknown test_idx: $test_idx!!!\n";
		}

		if($do_ns == 0 && $plot_types[$i] eq "energy") {
			print GP_OUT "set yrange [0:5.5]\n";
		} elsif($do_ns == 1 && $test_idx == 2 && $plot_types[$i] eq "energy") {
			print GP_OUT "set yrange [0:5.0]\n";
		} elsif($do_ns == 1 && $test_idx == 2 && $plot_types[$i] eq "pct_recv") {
			print GP_OUT "set yrange [0:1.2]\n";
		} elsif($do_ns == 1 && $test_idx == 0 && $plot_types[$i] eq "pct_recv") {
			print GP_OUT "set yrange [0:1.2]\n";
		} elsif($do_ns == 1 && $test_idx == 1 && $plot_types[$i] eq "pct_recv") {
			print GP_OUT "set yrange [0:1.2]\n";
		} elsif($do_ns == 0 && $plot_types[$i] eq "60-hop_hops") {
			print GP_OUT "set yrange [0:125]\n";
		} elsif($do_ns == 0 && $plot_types[$i] eq "20-hop_hops") {
			print GP_OUT "set yrange [0:50]\n";
		} else {
			print GP_OUT "set yrange [0:]\n";
		}

		if($do_pres == 1) {
			print GP_OUT "set xlabel\n";
			print GP_OUT "set ylabel\n";
		} else {
			print GP_OUT "set xlabel \"$x_labels[$test_idx]\"\n";
			print GP_OUT "set ylabel \"$y_labels{$plot_types[$i]}\"\n";
		}

		if($do_pres == 1) {
			print GP_OUT "set key off\n";
		} else {
			if($do_ns == 0 && $plot_types[$i] eq "recv_90") {
				print GP_OUT "set key bottom right \n";
			} elsif($do_ns == 0 && $plot_types[$i] eq "recv_99") {
				print GP_OUT "set key 0.16, 0.55\n";
			} elsif($do_ns == 1 && $test_idx == 2 && $plot_types[$i] eq "pct_recv") {
				print GP_OUT "set key bottom right\n";
			} elsif($do_ns == 1 && $test_idx == 2 && $plot_types[$i] eq "5-hop_lat") {
				print GP_OUT "set key bottom left\n";
			} elsif($do_ns == 1 && $test_idx == 2 && $plot_types[$i] eq "2-hop_lat") {
				print GP_OUT "set key bottom left\n";
			} elsif($do_ns == 1 && $test_idx == 0 && $plot_types[$i] eq "latency") {
				print GP_OUT "set key bottom 5, 20\n";
			} elsif($do_ns == 1 && $test_idx == 0 && $plot_types[$i] eq "pct_recv") {
				print GP_OUT "set key bottom right\n";
			} elsif($do_ns == 1 && $test_idx == 1 && $plot_types[$i] eq "pct_recv") {
				print GP_OUT "set key bottom right\n";
			} else {
				print GP_OUT "set key\n";
			}
		}

		if($do_eps == 1) {
			print GP_OUT "set size 0.6,0.6\n";
			print GP_OUT "set term postscript eps\n";
			print GP_OUT "set output \"$eps_fname\"\n";
			if(defined $titles{$plot_types[$i]} && $do_pres != 1) {
				print GP_OUT "set title \"$titles{$plot_types[$i]}\"\n";
			}
		}
		else {
			print GP_OUT "set term postscript\n";
			print GP_OUT "set output \"$ps_fname\"\n";
		}


		my %hash_vals;

		if($plot_types[$i] eq "energy") {
			%hash_vals = %e_vals;
		}
		elsif($plot_types[$i] eq "latency") {
			%hash_vals = %lat_vals;
		}
		elsif($plot_types[$i] eq "latency_perhop") {
			%hash_vals = %lat_perhop_vals;
		}
		elsif($plot_types[$i] eq "hops") {
			%hash_vals = %hops_vals;
		}
		elsif($plot_types[$i] eq "hops_perhop") {
			%hash_vals = %hops_perhop_vals;
		}
		elsif($plot_types[$i] eq "pct_recv") {
			%hash_vals = %update_pct_vals;
		}
		elsif($plot_types[$i] eq "all_recv") {
			%hash_vals = %all_recv_vals;
		}
		elsif($plot_types[$i] eq "recv_80") {
			%hash_vals = %recv_80_vals;
		}
		elsif($plot_types[$i] eq "recv_85") {
			%hash_vals = %recv_85_vals;
		}
		elsif($plot_types[$i] eq "recv_90") {
			%hash_vals = %recv_90_vals;
		}
		elsif($plot_types[$i] eq "recv_95") {
			%hash_vals = %recv_95_vals;
		}
		elsif($plot_types[$i] eq "recv_99") {
			%hash_vals = %recv_99_vals;
		}
		elsif($plot_types[$i] eq "recv_10") {
			%hash_vals = %recv_10_vals;
		}
		elsif($plot_types[$i] eq "nacks_sent") {
			%hash_vals = %nacks_sent_vals;
		}
		elsif($plot_types[$i] eq "bcasts_sent") {
			%hash_vals = %bcasts_sent_vals;
		}
		elsif($plot_types[$i] eq "pkts_sent") {
			%hash_vals = %pkts_sent_vals;
		}
		elsif($plot_types[$i] eq "pkts_recv") {
			%hash_vals = %pkts_recv_vals;
		}
		elsif($plot_types[$i] eq "bytes_sent") {
			%hash_vals = %bytes_sent_vals;
		}
		elsif($plot_types[$i] eq "bytes_recv") {
			%hash_vals = %bytes_recv_vals;
		}
		elsif($plot_types[$i] eq "zero_updates") {
			%hash_vals = %zero_updates_vals;
		}
		elsif(($tmp) = ($plot_types[$i] =~ /(\d+)-hop_lat/)) {
			%hash_vals = %{$hop_lat_vals{$tmp}};
			#print "\"$hop_lat_vals{$tmp}{psm}[0][0]\", \"$hash_vals{psm}[0][0]\"\n";
			#die "foo\n";
		}
		elsif(($tmp) = ($plot_types[$i] =~ /(\d+)-hop_hops/)) {
			%hash_vals = %{$hop_hops_vals{$tmp}};
		}
		#next;
	
		print GP_OUT "\n";
	
		my (@dat_lines);
		my $first_flag = 1;
		my $ctr = 0;
		my ($y_val, $y_err, $low_err, $high_err, $y_col, 
				$low_err_col, $high_err_col);
		# key = protocol, j = x_param
		foreach $key (keys(%hash_vals)){
			for($j = 0; $j < @{$hash_vals{$key}}; $j++) {
				if($first_flag == 1) {
					$dat_lines[$j] = "\n$x_param[$test_idx][$j] ";
					if($j == 0) {
						print GP_OUT "plot \\\n";
					}
				}

				$y_val = sprintf("%.9f", avg_wo_undef(@{$hash_vals{$key}[$j]}));
				$y_err = sprintf("%.9f", std_dev_wo_undef(@{$hash_vals{$key}[$j]}));
				$low_err = sprintf("%.9f", ($y_val - $y_err));
				$low_err = ($low_err > 0) ? $low_err : 0.0;
				$high_err = sprintf("%.9f", ($y_val + $y_err));

				$dat_lines[$j] = $dat_lines[$j]."$y_val $low_err $high_err ";

				if($j == 0) {
					$y_col = 3 * $ctr + 2;		
					$low_err_col = $y_col + 1;		
					$high_err_col = $y_col + 2;		

					my $every_str = "";
					if($do_ns == 0 && $plot_types[$i] eq "60-hop_hops" &&
							$key eq "p_0.75") {
						$every_str = " every ::3";
					} elsif($do_ns == 0 && $plot_types[$i] eq "60-hop_hops" &&
							$key eq "p_0.5") {
						$every_str = " every ::1";
					} elsif($do_ns == 0 && $plot_types[$i] eq "20-hop_hops" &&
							$key eq "p_0.75") {
						$every_str = " every ::2";
					}

					#print GP_OUT "\t\"$dat_fname\" using 1:$y_col ".
					#		"title '$prot_labels{$key}' w lp";
					$line_title = "title \'$prot_labels{$key}\'";
					if($do_pres == 1) {
						$line_title = "notitle";
					}

					# For the presentation, we want to remove some lines
					if($do_pres == 1 && $do_ns == 0 && 
							$plot_types[$i] eq "60-hop_hops" && $key eq "p_0.5") {
						next;
					} elsif ($do_pres == 1 && $do_ns == 0 && 
							$plot_types[$i] eq "latency_perhop" && 
							($key eq "p_0.5" || $key eq "p_0.25")) {
						next;
					}

					print GP_OUT "\t\"$dat_fname\" $every_str using 1:$y_col ".
							"$line_title w lp lt ".
							"$prot_lt{$key} pt $prot_pt{$key}";

					if($do_std_dev == 1) {
						print GP_OUT ", \\\n";
						print GP_OUT "\t\"$dat_fname\" using 1:$y_col:$low_err_col".
								":$high_err_col notitle w errorbars lt $err_lt ps 0";
					}

					#print "ctr: $ctr, keys-1: ".(keys(%hash_vals) - 1)."\n";
					if($ctr == (keys(%hash_vals) - 1)) { 
						print GP_OUT "\n\n";
					}
					else {
						print GP_OUT ", \\\n";
					}
				}
			}
			$first_flag = 0;
			$ctr++;
		}
	
		# Print dat lines
		for($j = 0; $j < @{$x_param[$test_idx]}; $j++) {
			print DAT_OUT "$dat_lines[$j]";
		}
	
		print DAT_OUT "\n\n";
	
		close(DAT_OUT);
		close(GP_OUT);
	
		system("gnuplot", $gp_fname);
	}

	@plot_types = @old_plot_types;
	%y_labels = %old_y_labels;

}



