/**
 * All new code in this distribution is Copyright 2005 by North Carolina
 * State University. All rights reserved. Redistribution and use in
 * source and binary forms are permitted provided that this entire
 * copyright notice is duplicated in all such copies, and that any
 * documentation, announcements, and other materials related to such
 * distribution and use acknowledge that the software was developed at
 * North Carolina State University, Raleigh, NC. No charge may be made
 * for copies, derivations, or distributions of this material without the
 * express written consent of the copyright holder. Neither the name of
 * the University nor the name of the author may be used to endorse or
 * promote products derived from this material without specific prior
 * written permission.
 *
 * IN NO EVENT SHALL THE NORTH CAROLINA STATE UNIVERSITY BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE NORTH CAROLINA STATE UNIVERSITY HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN
 * "AS IS" BASIS, AND THE NORTH CAROLINA STATE UNIVERSITY HAS NO
 * OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. "
 *
 */

/**
 * show_result.java
 *
 * Author: An Liu
 * Date: 09/15/2005
 */


// imports
import net.tinyos.message.*;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.*;
import java.security.NoSuchAlgorithmException;




public class show_result implements MessageListener
{
	public int cnt = 0;
    /**
     * Main driver.
     *
     * @param argv  arguments
     */
    public static void main(String [] argv)
    {
        // try to start get_result application, else report failure
        try
        {
            new show_result();
        }
        catch (Exception e)
        {
            System.err.println("Exception: " + e);
            e.printStackTrace();
        }
        
        try{
            Signature ecdsa = Signature.getInstance("ECDSA");
    	} catch (NoSuchAlgorithmException e){
            System.out.println("Doesn't support ECDSA.");
    	}
    }


    /**
     * Implicit constructor.  Connects to the SerialForwarder,
     * registers itself as a listener for DbgMsg's,
     * and starts listening.
     */

    public show_result() throws Exception 
    {
        // connect to the SerialForwarder running on the local mote
        MoteIF mote = new MoteIF((net.tinyos.util.Messenger) null);

        // prepare to listen for messages of type result 
        mote.registerListener(new test_msg(), this);

        // start listening to the mote
        mote.start();
        System.out.println("start\n");
    }
    
    public void pArray(short[] a, int index, int len)
    {
    	for (int i=len-1+index; i>=index; i--)
    	{
            if (a[i] < 16)
                System.out.print("0");
            System.out.print(Integer.toHexString(a[i]));
    	}
    	System.out.println();
    }

    //get big number from array
    public BigInteger get_bn(short[] a, int index, int len)
    {
    	BigInteger tmp;
    	
    	tmp = new BigInteger("0");
    	
    	for (int i=index; i<len+index; i++)
    	{
            tmp = tmp.shiftLeft(8);
            tmp = tmp.add(BigInteger.valueOf(a[i]));
    	}
    	System.out.println(tmp.toString(16));
    	
    	return tmp;
    }

    /**
     * Event for handling incoming result's.
     *
     * @param dstaddr   destination address
     * @param msg       received message
     */
    public void messageReceived(int dstaddr, Message msg) 
    {

        // process any result's received
        if (msg instanceof test_msg)
        {
            // cast message
            test_msg dmsg = (test_msg) msg;
            int len = 25;
            int numbytes = 22;
            int digit_bits = 16;
            
            //result of ECC operation
            if (dmsg.get_type() == 101)
            {
            	BigInteger x, y, interval;
            	
            	System.out.print("test ECC operation\n" +
            			"P0.x = ");
            	x = get_bn(dmsg.get_data(), 0, len);
            	System.out.print("P0.y = ");
            	y = get_bn(dmsg.get_data(), len, len);
            	System.out.print("interval = ");
            	interval = get_bn(dmsg.get_data(), 2*len, 4);
            	System.out.println("lasting time is: " +
                                   interval.longValue()/921.6/1024 + 
                                   " seconds");
                System.out.println();
            }
            
            if (dmsg.get_type() == 102)
            {
            	BigInteger r, s, sig, ver;

            	System.out.print("test ECDSA\n");
            	System.out.print("sig = ");
            	sig = get_bn(dmsg.get_data(), 50, 4);
            	System.out.println("time for signature is: " +
                                   sig.longValue()/921.6/1024 + 
                                   " seconds");
            	System.out.print("ver = ");
            	ver = get_bn(dmsg.get_data(), 54, 4);
            	System.out.println("time for verification is: " + 
                                   ver.longValue()/921.6/1024 + 
                                   " seconds");
            	System.out.println("verification passed");
                System.out.println();
            }
            
            if (dmsg.get_type() == 103)
            {
            	System.out.println("verification not passed!!!!!!!!!!!!");
            }
            
        }
        else
        {
            // report error
            System.out.println("Unknown message type received.");
        }
    }
}
