
#ifndef _SIMPLEPBBFMSG_H
#define _SIMPLEPBBFMSG_H

#define DEBUG
#define SEND_STATS

// All times in ms unless otherwise noted
enum {
	BCAST_SRC_ID = 0,
	INIT_SRC_ID = 0xFFFF,
	PSM_STATE = 3,
	NO_PSM_STATE = 0,
	//UINT32_RANGE = 4294967296LL, // Max value + 1, used for wraparound
	MAX_NUM_PKTS = 300,
	MAX_NUM_BCAST_SRC = 40, 
	TEST_RUN_TIME = (1 * 30000L), 
	SEND_STATS_DELAY = 1000, // 7000,
	FIRST_SEND_STATS_DELAY = 1000, // (5 * 30000L),
	SEND_STATS_JITTER = 1000,
	ACK_TIMEOUT = 200,
	FEEDBACK_TIME = 10000, // 20000
	FIRST_FEEDBACK_TIME = 10000, // (3 * 60000L),
	BCAST_INTERVAL = 2500, 
	//ROUTE_UPDATE_INTERVAL = 5, // in seconds
	INIT_SEND_INTERVAL = 100, 
	RETURN_TO_PSM_INTERVAL = (3 * 60000L), 
	FIRST_RETURN_TO_PSM_INTERVAL = (3 * 60000L), // (5 * 60000L),
	MAC_RETRANSMIT_LIMIT = 5,
	STATS_RETRANSMIT_LIMIT = 3,
	LOW_TX_POWER_LEVEL = 0x01,
	HIGH_TX_POWER_LEVEL = 0xFF,
	TIME_BUF_LEN = 128
};

enum {
	NO_CTRL = 0,
	SET_PQR_VALS = 1,
	TERMINATE_TEST = 2
};

/* Reasons for resending a packet */
enum {
	NO_RESEND = 0,
	R_VAL = 1	
};

#define TX_EVENT_STR "TX_EVENT"
#define RESEND_EVENT_STR "RESEND_EVENT"
#define RX_EVENT_STR "RX_EVENT"
#define FWD_EVENT_STR "FWD_EVENT"

#ifdef TOSSIM
	#define TO_MS_CONV (1000000.0/1000.0)
#else
	#define TO_MS_CONV (921600.0/1000.0)
#endif

enum {
	AM_UARTSIMPLEPBBFMSG = 77,
	AM_SIMPLEPBBFMSG = 78,
	AM_PBBFSTATSMSG = 79,
	AM_PBBFSTATSACK = 80 
};

// Put the uint16 first to try to
// get more compact alignment
typedef struct SimplePbbfMsg {
	uint16_t source;
	uint8_t seqno;
	uint8_t hopCount;
	uint8_t numFastHops;
	uint8_t ctrlMsgType;
	uint8_t pVal;
	uint8_t qVal;
	uint8_t rVal;
	uint8_t runSeqno;

	// Timestamp for stats
	// (Granularity = 1/921.6 kHz)
	uint32_t latency;
} __attribute__ ((packed)) SimplePbbfMsg;

typedef SimplePbbfMsg *SimplePbbfMsgPtr;

// NOTE: UartSimplePbbfMsg needs to
// be the exact same as SimplePbbfMsg.
// We need two types since both UART and radio packets
// go through GenericComm.  In order to use mig to 
// create the Java-side class, we have to have the 
// entire struct defined rather than just doing
// a typedef SimplePbbfMsg UartSimplePbbfMsg
typedef struct UartSimplePbbfMsg {
	uint16_t source;
	uint8_t seqno;
	uint8_t hopCount;
	uint8_t numFastHops;
	uint8_t ctrlMsgType;
	uint8_t pVal;
	uint8_t qVal;
	uint8_t rVal;
	uint8_t runSeqno;

	// Timestamp for stats
	// (Granularity = 1/921.6 kHz)
	uint32_t latency;
} __attribute__ ((packed)) UartSimplePbbfMsg;

typedef UartSimplePbbfMsg *UartSimplePbbfMsgPtr;

typedef struct PbbfStatsMsg {

	uint16_t nodeId;

	// NOTE: hopCount and totalFastHops only apply
	// to scenarios for which there is one broadcast source
	// Hop count from the source
	//uint16_t totalHopCountFromSrc;
	uint8_t hopCountFromSrc;

	// Hops where the sender immediately sent the packet
	uint16_t totalFastHops;

	uint16_t totalDataSent;
	uint16_t totalDataResent;

	uint16_t totalDataRecv;
	uint16_t totalAppRecv;

	uint32_t avgLat;

	uint8_t fracOnTime; // 0=0%, 255=100%, uniform spacing between

	uint8_t runSeqno;

	//uint16_t totalCtrlSent;
	//uint16_t totalCtrlRecv;

} __attribute__ ((packed)) PbbfStatsMsg;

typedef PbbfStatsMsg *PbbfStatsMsgPtr;

// Link layer ACKs seem to have a non-trivial
// false positive rate (the same ACK code is used
// without specifying the destination adddress).
// So, we'll do app layer ACKs as well.
typedef struct PbbfStatsAck {
	uint16_t ackSender;
	uint16_t ackDest;
} __attribute__ ((packed)) PbbfStatsAck;

typedef PbbfStatsAck *PbbfStatsAckPtr;

typedef struct NodeInfo {
	uint16_t addr;
	uint8_t seqno;
} NodeInfo;

typedef NodeInfo *NodeInfoPtr;

#endif

