
package net.tinyos.simple_pbbf;

import net.tinyos.util.*;
import net.tinyos.message.*;
import java.io.*;
import java.net.*;
import java.util.*;
//import java.awt.event.*;

public class PbbfComm implements MessageListener {	

	private static final String STATS_PRE = "PbbfComm";

	private static final int NO_CTRL = 0;
	private static final int SET_PQR_VALS = 1;
	private static final int TERMINATE_TEST = 2;

	private static final int TOS_UART_ADDR = 0x7E;

	// TIMEOUT is in ms
	private static final int TIMEOUT = ((1000 * 60) * 30);

	Timer timer_;

	short pVal_, qVal_, rVal_, runSeqno_;
	String fileName_;
	int seed_;

	MoteIF mote_;

	Vector nodeStats_;
    
	public static void main(String[] args) throws IOException {
		new PbbfComm(args);
	}

	public PbbfComm(String args[]) {

		pVal_ = 0;
		qVal_ = 0;
		rVal_ = 0;
		seed_ = 2006;
		runSeqno_ = 1;
		fileName_ = new String();
		boolean fileNameWasSet = false;

		nodeStats_ = new Vector();

	try {
		// Parse the input options
		for(int i = 0; i < args.length; i++) {
			if(args[i].equals("-help") || args[i].equals("-h")) {
				help(null);
			} else if(args[i].equals("-pVal")) {
				pVal_ = Short.parseShort(args[i+1]);
				i++;	
			} else if(args[i].equals("-qVal")) {
				qVal_ = Short.parseShort(args[i+1]);
				i++;	
			} else if(args[i].equals("-rVal")) {
				rVal_ = Short.parseShort(args[i+1]);
				i++;	
			} else if(args[i].equals("-seqno")) {
				runSeqno_ = Short.parseShort(args[i+1]);
				i++;	
			} else if(args[i].equals("-seed")) {
				seed_ = Integer.parseInt(args[i+1]);
				i++;
			} else if(args[i].equals("-file")) {
				fileName_ = args[i+1];
				fileNameWasSet = true;
				i++;
			} else {
				help("unrecongized option: " + args[i]);
			}
		}

			if(!fileNameWasSet) {
				fileName_ = STATS_PRE + 
				"-p" + pVal_ + "-q" + qVal_ + "-r" + rVal_ + 
				"-seqno" + runSeqno_ + "-seed" + seed_ + ".stats";
			}

	} catch (Exception e) {
		e.printStackTrace();
		help("exception parsing args: " + e);
		return;
	}

		// Connect to the serial forwarder and listen for pkts
		mote_ = new MoteIF();
		mote_.registerListener(new PbbfStatsMsg(), this);
		mote_.registerListener(new UartSimplePbbfMsg(), this);

		// Send the parameters to the mote and let TinyOS
		// do its thing until we receive a terminate signal
		sendParameters();

		// Start a timer to kill this program just in case
		// we never get a terminate signal from the mote
		timer_ = new Timer();
		timer_.schedule(new ExitTask(), TIMEOUT);

	}

	private class ExitTask extends TimerTask {
		public void run() {
			System.exit(1);
		}
	}

	private void sendParameters() {

		UartSimplePbbfMsg msg = new UartSimplePbbfMsg();
		//SimplePbbfMsg msg = new SimplePbbfMsg();

		msg.set_source(TOS_UART_ADDR);
		msg.set_seqno(runSeqno_);
		msg.set_hopCount((short)0);
		msg.set_numFastHops((short)0);
		msg.set_ctrlMsgType((short)SET_PQR_VALS);
		msg.set_pVal(pVal_);
		msg.set_qVal(qVal_);
		msg.set_rVal(rVal_);
		msg.set_runSeqno(runSeqno_);

	try {
		System.err.println("sendMsg");
		System.err.println(msg.toString());
		mote_.send(MoteIF.TOS_BCAST_ADDR, msg);
	} catch (IOException ioe) {
		System.err.println(
			"Warning: Got IOException sending PQR message: "+ioe);
		ioe.printStackTrace();
	}


	}

	// We got a message from the mote
	public void messageReceived(int destAddr, Message msg) {
		System.err.println("msgRecv");
		if (msg instanceof PbbfStatsMsg) {
			pbbfStatsReceived(destAddr, (PbbfStatsMsg)msg);
		} else if (msg instanceof SimplePbbfMsg) {
			System.err.println("Received SimplePbbfMsg packet " +
					"instead of UartSimplePbbfMsg.");
		} else if (msg instanceof UartSimplePbbfMsg) {
			pbbfMsgReceived(destAddr, (UartSimplePbbfMsg)msg);
		} else {
			throw new RuntimeException(
				"messageReceived: Got bad message type: "+msg);
		}
	}

	private void pbbfStatsReceived(int destAddr, 
			PbbfStatsMsg msg) {

		NodeStats ns = new NodeStats();

		System.err.println(msg.toString());

		// Compare seqno with runSeqno
		if(msg.get_runSeqno() == runSeqno_) {

			// Duplicates may occur due to the ACKing scheme,
			// do dup supression here.
			NodeStats tmp_ns;
			int i;
			for(i = 0; i < nodeStats_.size(); i++) {
				tmp_ns = (NodeStats)nodeStats_.elementAt(i);
				if(tmp_ns.getNodeId() == msg.get_nodeId())
					break;
			}

			if(i == nodeStats_.size()) {

				// Collect the received stats in our array
				ns.setNodeId(msg.get_nodeId());
				ns.setHopCountFromSrc(msg.get_hopCountFromSrc());
				ns.setTotalFastHops(msg.get_totalFastHops());
				ns.setTotalDataSent(msg.get_totalDataSent());
				ns.setTotalDataResent(msg.get_totalDataResent());
				ns.setTotalDataRecv(msg.get_totalDataRecv());
				ns.setTotalAppRecv(msg.get_totalAppRecv());
				ns.setAvgLat(msg.get_avgLat());
				ns.setFracOnTime(msg.get_fracOnTime());
				ns.setRunSeqno(msg.get_runSeqno());
				//ns.setTotalCtrlSent(msg.get_totalCtrlSent());
				//ns.setTotalCtrlRecv(msg.get_totalCtrlRecv());
		
				nodeStats_.add(ns);
			}

		}

	}

	private void pbbfMsgReceived(int destAddr, UartSimplePbbfMsg msg) {

		switch(msg.get_ctrlMsgType()) {
			case TERMINATE_TEST:
				// Write stats to file and exit
				int exitVal = writeStatsToFile();
				System.exit(exitVal);
				break;
			default:
				System.err.println("Unknown pbbfMsgReceived: " + 
						msg.get_ctrlMsgType());
				break;
		}
	}

	private int writeStatsToFile() {

		int retVal = 1;

		// Sort our list based on node id
		Collections.sort(nodeStats_, new NodeIdComp());

	try {
		// Name the file based on the parameters
		PrintStream out = 
				new PrintStream(new FileOutputStream(fileName_));

		out.println("");
		out.println("Stats for P=" + pVal_ + ", Q=" + qVal_ +
				", R=" + rVal_ + " (seed=" + seed_ + 
				", seqno=" + runSeqno_ + ")");
		out.println("");
		NodeStats ns;
		for(int i = 0; i < nodeStats_.size(); i++) {
			ns = (NodeStats)nodeStats_.elementAt(i);
			out.println("Node " + ns.getNodeId());
			out.println("\tHopCountFromSrc " + 
					ns.getHopCountFromSrc());
			out.println("\tTotalFastHops " + 
					ns.getTotalFastHops());
			out.println("\tTotalDataSent " + 
					ns.getTotalDataSent());
			out.println("\tTotalDataResent " + 
					ns.getTotalDataResent());
			out.println("\tTotalDataRecv " + 
					ns.getTotalDataRecv());
			out.println("\tTotalAppRecv " + 
					ns.getTotalAppRecv());
			out.println("\tAvgLat " + 
					ns.getAvgLat());
			out.println("\tFracOnTime " + 
					ns.getFracOnTime());
			out.println("\tRunSeqno " + 
					ns.getRunSeqno());
			/*
			out.println("\tTotalCtrlSent " + 
					ns.getTotalCtrlSent());
			out.println("\tTotalCtrlRecv " + 
					ns.getTotalCtrlRecv());
			*/
		}
		out.println("");
		out.close();

	} catch (IOException ioe) {
		System.err.println(
			"Warning: Got IOException opening output file: "+ioe);
		ioe.printStackTrace();

	}

		return retVal;
	}

	private void help(String error) {
		int exitStatus = 0;
		if(error != null) {
			System.err.println("Error parsing args (" + 
				error + ")\n");
			exitStatus = 1;
		}
		System.err.println("PbbfComm usage:");
		System.err.println("\tjava net.tinyos.simple_pbbf.PbbfComm" +
			" [options]");
		System.err.println("");
		System.err.println("Options:");
		System.err.println("\t-pVal p value (ints 0 to 10)");
		System.err.println("\t-qVal q value (ints 0 to 10)");
		System.err.println("\t-rVal r value (ints 0 to 10)");
		System.err.println("\t-seqno sequence number of the test");
		System.err.println("\t-seed random seed");
		System.err.println("\t-file output file name");
		System.err.println("");
		System.exit(exitStatus);
	}

} // end PbbfComm class

class NodeIdComp implements Comparator {
	public int compare (Object o1, Object o2) {
		int n1 = ((NodeStats)o1).getNodeId();
		int n2 = ((NodeStats)o2).getNodeId();
		return (Math.abs(n1) - Math.abs(n2));
	}
} // end NodeIdComp class

class NodeStats {
	
	int nodeId_;
	int hopCountFromSrc_;
	int totalFastHops_;
	int totalDataSent_;
	int totalDataResent_;
	int totalDataRecv_;
	int totalAppRecv_;	
	long avgLat_;
	short fracOnTime_;
	short runSeqno_;
	//int totalCtrlSent_;
	//int totalCtrlRecv_;

	public NodeStats() {
		nodeId_ = -1;
		hopCountFromSrc_ = -1;
		totalFastHops_ = -1;
		totalDataSent_ = -1;
		totalDataResent_ = -1;
		totalDataRecv_ = -1;
		totalAppRecv_ = -1;	
		avgLat_ = -1;
		fracOnTime_ = -1;
		runSeqno_ = -1;
		//totalCtrlSent_ = -1;
		//totalCtrlRecv_ = -1;
	}

	public int getNodeId() {
		return nodeId_;
	}

	public void setNodeId(int v) {
		nodeId_ = v;
	}

	public int getHopCountFromSrc() {
		return hopCountFromSrc_;
	}

	public void setHopCountFromSrc(int v) {
		hopCountFromSrc_ = v;
	}

	public int getTotalFastHops() {
		return totalFastHops_;
	}

	public void setTotalFastHops(int v) {
		totalFastHops_ = v;
	}

	public int getTotalDataSent() {
		return totalDataSent_;
	}

	public void setTotalDataSent(int v) {
		totalDataSent_ = v;
	}

	public int getTotalDataResent() {
		return totalDataResent_;
	}

	public void setTotalDataResent(int v) {
		totalDataResent_ = v;
	}

	public int getTotalDataRecv() {
		return totalDataRecv_;
	}

	public void setTotalDataRecv(int v) {
		totalDataRecv_ = v;
	}

	public int getTotalAppRecv() {
		return totalAppRecv_;
	}

	public void setTotalAppRecv(int v) {
		totalAppRecv_ = v;
	}

	public long getAvgLat() {
		return avgLat_;
	}

	public void setAvgLat(long v) {
		avgLat_ = v;
	}

	public short getFracOnTime() {
		return fracOnTime_;
	}

	public void setFracOnTime(short v) {
		fracOnTime_ = v;
	}

	public short getRunSeqno() {
		return runSeqno_;
	}

	public void setRunSeqno(short v) {
		runSeqno_ = v;
	}

	/*
	public int getTotalCtrlSent() {
		return totalCtrlSent_;
	}

	public void setTotalCtrlSent(int v) {
		totalCtrlSent_ = v;
	}

	public int getTotalCtrlRecv() {
		return totalCtrlRecv_;
	}

	public void setTotalCtrlRecv(int v) {
		totalCtrlRecv_ = v;
	}
	*/

} // end NodeStats class


